@@ -1,12 +1,12 @@
 /*
-** Copyright (C) 1997-2012,2017 Free Software Foundation, Inc.
+** Copyright 2017-2019,2020 Thomas E. Dickey
+** Copyright 1997-2012,2017 Free Software Foundation, Inc.
 **
 ** This file is part of TACK.
 **
 ** TACK is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2, or (at your option)
-** any later version.
+** the Free Software Foundation, version 2.
 **
 ** TACK is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -24,15 +24,11 @@
 
 #include <tack.h>
 
-MODULE_ID("$Id: edit.c,v 1.38 2017/07/26 08:46:44 tom Exp $")
+MODULE_ID("$Id: edit.c,v 1.48 2020/02/14 21:13:30 tom Exp $")
 
 /*
  * These are adapted from tic.h
  */
-#define ABSENT_STRING		(char *)0
-#define CANCELLED_STRING	(char *)(-1)
-#define VALID_STRING(s)  ((s) != CANCELLED_STRING && (s) != ABSENT_STRING)
-
 typedef struct {
     const char *nt_name;
     NAME_TYPE nt_type;
@@ -44,7 +40,7 @@ typedef struct {
 /*
  * Terminfo edit features
  */
-#ifdef NCURSES_VERSION
+#if TACK_CAN_EDIT
 
 #define SHOW_VALUE	1
 #define SHOW_EDIT	2
@@ -66,7 +62,7 @@ static int *label_strings;
 static int xon_index;		/* Subscript for (xon) */
 static int xon_shadow;
 
-#ifdef NCURSES_VERSION
+#if TACK_CAN_EDIT
 static int start_display;	/* the display has just started */
 static int display_lines;	/* number of lines displayed */
 #endif
@@ -94,7 +90,7 @@ compare_capability(const void *a, const void *b)
     return strcmp(p->nt_name, q->nt_name);
 }
 
-#ifdef HAVE_CURSES_DATA_BOOLNAMES
+#if (defined(HAVE_CURSES_DATA_BOOLNAMES) || defined(DECL_CURSES_DATA_BOOLNAMES))
 
 #define DATA(index,name,type) { name,type,index }
 static NAME_TABLE name_table[] =
@@ -130,19 +126,19 @@ alloc_name_table(void)
 	for (s = 0; s < max_booleans; ++s) {
 	    name_table[d].nt_name = boolnames[s];
 	    name_table[d].nt_type = BOOLEAN;
-	    name_table[d].nt_index = s;
+	    name_table[d].nt_index = (int) s;
 	    ++d;
 	}
 	for (s = 0; s < max_numbers; ++s) {
 	    name_table[d].nt_name = numnames[s];
 	    name_table[d].nt_type = NUMBER;
-	    name_table[d].nt_index = s;
+	    name_table[d].nt_index = (int) s;
 	    ++d;
 	}
 	for (s = 0; s < max_strings; ++s) {
 	    name_table[d].nt_name = strnames[s];
 	    name_table[d].nt_type = STRING;
-	    name_table[d].nt_index = s;
+	    name_table[d].nt_index = (int) s;
 	    ++d;
 	}
 	qsort(name_table, sizeof_name_table, sizeof(NAME_TABLE), compare_data);
@@ -167,7 +163,7 @@ find_capability(const char *name)
     return lookup ? lookup : 0;
 }
 
-#ifndef NCURSES_VERSION
+#if !TACK_CAN_EDIT
 /*
  * This is used to relate array-index to capability name/type.
  */
@@ -198,7 +194,7 @@ find_string_cap_by_name(const char *name)
     return result;
 }
 
-#ifdef NCURSES_VERSION
+#if TACK_CAN_EDIT
 
 #define set_saved_boolean(num, value) original_term.Booleans[num] = (char)value
 #define set_saved_number(num, value)  original_term.Numbers[num]  = (short)value
@@ -293,9 +289,9 @@ int
 get_string_cap_byvalue(
 			  const char *value)
 {
-    int i;
-
     if (value) {
+	int i;
+
 	for (i = 0; i < (int) MAX_STRINGS; i++) {
 	    /* FIXME - this implies ncurses... */
 	    if (get_newer_string(i) == value) {
@@ -317,15 +313,14 @@ get_string_cap_byvalue(
 **
 **	Return TRUE if the user has modified the terminfo
 */
-#ifdef NCURSES_VERSION
+#if TACK_CAN_EDIT
 int
 user_modified(void)
 {
-    const char *a, *b;
-    int i, v;
+    int i;
 
     for (i = 0; i < MAX_BOOLEAN; i++) {
-	v = (i == xon_index) ? xon_shadow : get_newer_boolean(i);
+	int v = (i == xon_index) ? xon_shadow : get_newer_boolean(i);
 	if (get_saved_boolean(i) != v) {
 	    return TRUE;
 	}
@@ -336,6 +331,7 @@ user_modified(void)
 	}
     }
     for (i = 0; i < (int) MAX_STRINGS; i++) {
+	const char *a, *b;
 	if ((a = get_saved_string(i)) == 0)
 	    a = "";
 	if ((b = get_newer_string(i)) == 0)
@@ -385,13 +381,13 @@ mark_cap(
 					   | flag));
 	    break;
 	default:
-	    sprintf(temp, "unknown cap type (%s)", name);
+	    sprintf(temp, "unknown cap type (%.*s)", NAME_SIZE, name);
 	    ptextln(temp);
 	    break;
 	}
     } else {
 #ifdef HAVE_CURSES_DATA_BOOLNAMES
-	sprintf(temp, "Cap not found: %s", name);
+	sprintf(temp, "Cap not found: %.*s", NAME_SIZE, name);
 	ptextln(temp);
 	(void) wait_here();
 #endif
@@ -410,10 +406,10 @@ can_test(
 	    const char *s,
 	    int flags)
 {
-    int ch, j;
-    char name[32];
-
     if (s) {
+	int ch, j;
+	char name[NAME_SIZE];
+
 	for (j = 0; (ch = name[j] = *s); s++) {
 	    if (ch == ' ' || ch == ')' || ch == '(') {
 		if (j) {
@@ -443,15 +439,15 @@ cap_index(
 	     const char *s,
 	     int *inx)
 {
-    NAME_TABLE const *nt;
-    int ch, j;
-    char name[32];
-
     if (s) {
+	int j;
+	char name[NAME_SIZE];
+
 	for (j = 0;; s++) {
-	    ch = name[j] = *s;
+	    int ch = name[j] = *s;
 	    if (ch == ' ' || ch == ')' || ch == '(' || ch == 0) {
 		if (j) {
+		    NAME_TABLE const *nt;
 		    name[j] = '\0';
 		    if ((nt = find_string_cap_by_name(name)) != 0) {
 			*inx++ = nt->nt_index;
@@ -481,14 +477,13 @@ cap_match(
 	     const char *names,
 	     const char *cap)
 {
-    const char *s;
-    int c, l, t;
-
     if (names) {
-	l = (int) strlen(cap);
+	int l = (int) strlen(cap);
+	const char *s;
+
 	while ((s = strstr(names, cap))) {
-	    c = (names == s) ? 0 : *(s - 1);
-	    t = s[l];
+	    int c = (names == s) ? 0 : *(s - 1);
+	    int t = s[l];
 	    if ((c == 0 || c == ' ' || c == '(') &&
 		(t == 0 || t == ' ' || t == ')')) {
 		return TRUE;
@@ -564,6 +559,7 @@ show_report(
 }
 
 #ifdef NCURSES_VERSION
+#if TACK_CAN_EDIT
 static size_t
 safe_length(const char *value)
 {
@@ -595,8 +591,8 @@ copy_termtype(TERMTYPE *target, TERMTYPE *source)
     memset(target, 0, sizeof(*target));
 
 #define copy_array(member,count) \
-    target->member = calloc(count, sizeof(target->member[0])); \
-    memcpy(target->member, source->member, count * sizeof(target->member[0]))
+    target->member = calloc((size_t)(count), sizeof(target->member[0])); \
+    memcpy(target->member, source->member, (size_t)(count) * sizeof(target->member[0]))
 
     copy_array(Booleans, MAX_BOOLEAN);
     copy_array(Numbers, MAX_NUMBERS);
@@ -652,6 +648,7 @@ copy_termtype(TERMTYPE *target, TERMTYPE *source)
     }
 #endif /* NCURSES_XNAMES */
 }
+#endif /* TACK_CAN_EDIT */
 
 #if NO_LEAKS
 static void
@@ -684,7 +681,7 @@ edit_init(void)
 
     alloc_arrays();
 
-#ifdef NCURSES_VERSION
+#if TACK_CAN_EDIT
     copy_termtype(&original_term, CUR_TP);
     for (i = 0; i < MAX_BOOLEAN; i++) {
 	set_saved_boolean(i, get_newer_boolean(i));
@@ -736,7 +733,7 @@ edit_init(void)
     FreeIfNeeded(label_strings);
 }
 
-#ifdef NCURSES_VERSION
+#if TACK_CAN_EDIT
 
 /*
 **	save_info_string(str, fp)
@@ -801,7 +798,9 @@ form_terminfo(const char *srcp)
 	return 0;
     }
     if (buffer == 0 || need > length) {
+	char *tofree = buffer;
 	if ((buffer = (char *) realloc(buffer, length = need)) == 0) {
+	    free(tofree);
 	    return 0;
 	}
     }
@@ -827,8 +826,7 @@ form_terminfo(const char *srcp)
 		    && isprint((int) value)) {
 		    ch = (int) value;
 		    buffer[bufp++] = S_QUOTE;
-		    if (ch == '\\'
-			|| ch == S_QUOTE)
+		    if (ch == S_QUOTE)
 			buffer[bufp++] = '\\';
 		    buffer[bufp++] = (char) ch;
 		    buffer[bufp++] = S_QUOTE;
@@ -896,7 +894,7 @@ save_info(
     int i;
     FILE *fp;
     time_t now;
-    char buf[1024];
+    char buf[TEMP_SIZE];
 
     if ((fp = fopen(tty_basename, "w")) == (FILE *) NULL) {
 	(void) sprintf(temp, "can't open: %s", tty_basename);
@@ -997,7 +995,7 @@ show_info(
 	     int *ch)
 {
     int i;
-    char buf[1024];
+    char buf[TEMP_SIZE];
 
     display_lines = 1;
     start_display = 1;
@@ -1184,9 +1182,9 @@ show_value(
 {
     NAME_TABLE const *nt;
     char *s;
-    int n, op, b;
-    char buf[1024];
-    char tmp[1024];
+    int n, op;
+    char buf[TEMP_SIZE];
+    char tmp[TEMP_SIZE];
 
     ptext("enter name: ");
     read_string(buf, (size_t) 80);
@@ -1198,7 +1196,10 @@ show_value(
 	put_clear();
     }
     op = t->flags & 255;
+
     if ((nt = find_capability(buf)) != 0) {
+	int b;
+
 	switch (nt->nt_type) {
 	case BOOLEAN:
 	    if (op == SHOW_DELETE) {
@@ -1212,7 +1213,7 @@ show_value(
 	    b = ((nt->nt_index == xon_index)
 		 ? xon_shadow
 		 : get_newer_boolean(nt->nt_index));
-	    sprintf(temp, "boolean  %s %s", buf,
+	    sprintf(temp, "boolean  %.*s %s", (TEMP_SIZE - 30), buf,
 		    b ? "True" : "False");
 	    break;
 	case STRING:
@@ -1221,10 +1222,10 @@ show_value(
 		return;
 	    }
 	    if (get_newer_string(nt->nt_index)) {
-		sprintf(temp, "string  %s %s", buf,
+		sprintf(temp, "string  %.*s %s", NAME_SIZE, buf,
 			expand(get_newer_string(nt->nt_index)));
 	    } else {
-		sprintf(temp, "undefined string %s", buf);
+		sprintf(temp, "undefined string %.*s", NAME_SIZE, buf);
 	    }
 	    break;
 	case NUMBER:
@@ -1232,7 +1233,7 @@ show_value(
 		set_newer_number(nt->nt_index, -1);
 		return;
 	    }
-	    sprintf(temp, "numeric  %s %d", buf,
+	    sprintf(temp, "numeric  %.*s %d", NAME_SIZE, buf,
 		    get_newer_number(nt->nt_index));
 	    break;
 	default:
@@ -1241,7 +1242,7 @@ show_value(
 	}
 	ptextln(temp);
     } else {
-	sprintf(temp, "Cap not found: %s", buf);
+	sprintf(temp, "Cap not found: %.*s", NAME_SIZE, buf);
 	ptextln(temp);
 	return;
     }
@@ -1278,7 +1279,7 @@ show_value(
 		    nt->nt_name, n);
 	    ptextln(temp);
 	} else {
-	    sprintf(temp, "Illegal number: %s", buf);
+	    sprintf(temp, "Illegal number: %.*s", (TEMP_SIZE - 20), buf);
 	    ptextln(temp);
 	}
 	break;
@@ -1337,14 +1338,12 @@ show_changed(
 		int *state GCC_UNUSED,
 		int *ch)
 {
-    int i, header = 1, v;
-    const char *a;
-    const char *b;
+    int i, header = 1;
     static char title[] = "                     old value   cap  new value";
-    char abuf[1024];
+    char abuf[TEMP_SIZE];
 
     for (i = 0; i < MAX_BOOLEAN; i++) {
-	v = (i == xon_index) ? xon_shadow : get_newer_boolean(i);
+	int v = (i == xon_index) ? xon_shadow : get_newer_boolean(i);
 	if (get_saved_boolean(i) != v) {
 	    if (header) {
 		ptextln(title);
@@ -1368,6 +1367,8 @@ show_changed(
 	}
     }
     for (i = 0; i < (int) MAX_STRINGS; i++) {
+	const char *a;
+	const char *b;
 	if ((a = get_saved_string(i)) == 0)
 	    a = "";
 	if ((b = get_newer_string(i)) == 0)
@@ -1377,8 +1378,9 @@ show_changed(
 		ptextln(title);
 		header = 0;
 	    }
-	    strcpy(abuf, form_terminfo(a));
-	    sprintf(temp, "%30s %6s %s", abuf, STR_NAME(i), form_terminfo(b));
+	    sprintf(abuf, "%.*s", (int) sizeof(abuf) - 1, form_terminfo(a));
+	    sprintf(temp, "%.30s %6s %.30s",
+		    abuf, STR_NAME(i), form_terminfo(b));
 	    putln(temp);
 	}
     }
@@ -1405,14 +1407,14 @@ change_one_entry(
     const char *s;
     char *t, *p;
     const char *current_string;
-    char buf[1024];
-    char pad[1024];
+    char buf[TEMP_SIZE];
+    char pad[TEMP_SIZE];
 
     i = test->flags & 255;
     if (i == 255) {
 	/* read the cap name from the user */
 	ptext("enter name: ");
-	read_string(pad, (size_t) 32);
+	read_string(pad, (size_t) NAME_SIZE);
 	if (pad[0] == '\0' || pad[1] == '\0') {
 	    *chp = pad[0];
 	    return;
@@ -1421,7 +1423,7 @@ change_one_entry(
 	    x = nt->nt_index;
 	    current_string = get_newer_string(x);
 	} else {
-	    sprintf(temp, "%s is not a string capability", pad);
+	    sprintf(temp, "%.*s is not a string capability", NAME_SIZE, pad);
 	    ptext(temp);
 	    generic_done_message(test, state, chp);
 	    return;
@@ -1429,7 +1431,7 @@ change_one_entry(
     } else {
 	x = tx_index[i];
 	current_string = tx_cap[i];
-	strcpy(pad, STR_NAME(x));
+	sprintf(pad, "%.*s", (int) sizeof(pad) - 1, STR_NAME(x));
     }
     if (!current_string) {
 	ptextln("That string is not currently defined.  Please enter a new value, including the padding delay:");
@@ -1443,12 +1445,12 @@ change_one_entry(
 	ptextln(expand(t));
 	return;
     }
-    sprintf(buf, "Current value: (%s) %s",
-	    pad,
+    sprintf(buf, "Current value: (%.*s) %s",
+	    NAME_SIZE, pad,
 	    form_terminfo(current_string));
     putln(buf);
     ptextln("Enter new pad.  0 for no pad.  CR for no change.");
-    read_string(buf, (size_t) 32);
+    read_string(buf, (size_t) NAME_SIZE);
     if (buf[0] == '\0' || (buf[1] == '\0' && isalpha(UChar(buf[0])))) {
 	*chp = buf[0];
 	return;
@@ -1499,7 +1501,7 @@ change_one_entry(
 	v = ch;
     }
     if (pad[0]) {
-	sprintf(t, "$<%s>", pad);
+	sprintf(t, "$<%.*s>", 20, pad);
     }
     if ((t = (char *) malloc(strlen(buf) + 1))) {
 	strcpy(t, buf);
@@ -1520,10 +1522,11 @@ static void
 build_change_menu(
 		     TestMenu * m)
 {
-    int i, j, k;
+    int i, j;
     char *s;
 
     for (i = j = 0; i < txp; i++) {
+	int k;
 	if ((k = tx_index[i]) >= 0) {
 	    s = form_terminfo(tx_cap[i]);
 	    s[40] = '\0';
@@ -1572,7 +1575,7 @@ TestMenu change_pad_menu =
 };
 
 #else
-#endif /* NCURSES_VERSION */
+#endif /* TACK_CAN_EDIT */
 
 #if NO_LEAKS
 void
