@@ -1,12 +1,12 @@
 /*
-** Copyright (C) 1991, 1997-2013,2017 Free Software Foundation, Inc.
+** Copyright 2017-2021,2022 Thomas E. Dickey
+** Copyright 1997-2013,2017 Free Software Foundation, Inc.
 **
 ** This file is part of TACK.
 **
 ** TACK is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2, or (at your option)
-** any later version.
+** the Free Software Foundation, version 2.
 **
 ** TACK is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -21,7 +21,9 @@
 
 #include <tack.h>
 
-MODULE_ID("$Id: fun.c,v 1.24 2017/07/23 16:08:17 tom Exp $")
+MODULE_ID("$Id: fun.c,v 1.32 2022/05/28 13:46:42 tom Exp $")
+
+#define COPY_1(target, source) sprintf(target, "%.*s", (int)sizeof(target) - 1, source)
 
 /*
  * Test the function keys on the terminal.  The code for echo tests
@@ -45,7 +47,6 @@ TestList funkey_test_list[] = {
     {MENU_LAST, 0, 0, 0, 0, 0, 0}
 };
 /* *INDENT-ON* */
-
 static void printer_on(TestList *, int *, int *);
 static void printer_mc0(TestList *, int *, int *);
 
@@ -112,7 +113,7 @@ keys_tested(
 	putln("Function key labels:");
 	for (i = 0; i < key_count; ++i) {
 	    if (fk_label[i]) {
-		sprintf(outbuf, "%s %s",
+		sprintf(outbuf, "%.80s %.80s",
 			fk_name[i] ? fk_name[i] : "??", fk_label[i]);
 		put_columns(outbuf, (int) strlen(outbuf), 16);
 	    }
@@ -136,15 +137,15 @@ keys_tested(
 	for (i = 0; scan_down[i]; i++) {
 	    if (!scan_tested[i]) {
 		if (hex_output) {
-		    strcpy(outbuf, hex_expand_to(scan_down[i], 3));
+		    COPY_1(outbuf, hex_expand_to(scan_down[i], 3));
 		} else {
-		    strcpy(outbuf, expand(scan_down[i]));
+		    COPY_1(outbuf, expand(scan_down[i]));
 		}
 		l = expand_chars;
 		if (hex_output) {
-		    strcat(outbuf, hex_expand_to(scan_up[i], 3));
+		    COPY_1(outbuf, hex_expand_to(scan_up[i], 3));
 		} else {
-		    strcat(outbuf, expand(scan_up[i]));
+		    COPY_1(outbuf, expand(scan_up[i]));
 		}
 		expand_chars += l;
 		l = (int) strlen(scan_name[i]);
@@ -164,9 +165,9 @@ keys_tested(
 	for (i = 0; i < key_count; i++) {
 	    if (!fk_tested[i] && fk_name[i] != 0) {
 		if (hex_output) {
-		    strcpy(outbuf, hex_expand_to(fkval[i], 3));
+		    COPY_1(outbuf, hex_expand_to(fkval[i], 3));
 		} else {
-		    strcpy(outbuf, expand(fkval[i]));
+		    COPY_1(outbuf, expand(fkval[i]));
 		}
 		l = (int) strlen(fk_name[i]);
 		if (((char_count + 16) & ~15) +
@@ -200,13 +201,12 @@ enter_key(
 	     char *value,
 	     char *lab)
 {
-    size_t j;
 
     alloc_strings();
     if (value) {
 	int found = 0;
 	if (key_count != 0) {
-	    j = strlen(value);
+	    size_t j = strlen(value);
 	    fkmax = (fkmax > j) ? fkmax : j;
 	    /* do not permit duplicates */
 	    for (j = 0; j < (size_t) key_count; j++) {
@@ -288,7 +288,6 @@ static int
 found_match(char *s, int hx, int cc)
 {				/* return true if this string is a match */
     int j, f;
-    char outbuf[256];
 
     alloc_strings();
     if (!*s) {
@@ -346,7 +345,9 @@ found_match(char *s, int hx, int cc)
 	}
     } else {
 	for (j = f = 0; j < key_count; j++) {
-	    if (!strcmp(s, fkval[j])) {
+	    if (fkval[j] && !strcmp(s, fkval[j])) {
+		char outbuf[256];
+
 		if (!f) {	/* first match */
 		    put_cr();
 		    if (hx) {
@@ -356,10 +357,10 @@ found_match(char *s, int hx, int cc)
 		    }
 		    f = 1;
 		}
-		sprintf(outbuf, " (%s)", fk_name[j]);
+		sprintf(outbuf, " (%.80s)", fk_name[j]);
 		put_str(outbuf);
 		if (fk_label[j]) {
-		    sprintf(outbuf, " <%s>", fk_label[j]);
+		    sprintf(outbuf, " <%.80s>", fk_label[j]);
 		    put_str(outbuf);
 		}
 		fk_tested[j] = 1;
@@ -377,7 +378,7 @@ found_match(char *s, int hx, int cc)
 static int
 found_exit(char *keybuf, int hx, int cc)
 {				/* return true if the user wants to exit */
-    int j, k;
+    int j;
     char *s;
 
     if (scan_mode) {
@@ -385,6 +386,8 @@ found_exit(char *keybuf, int hx, int cc)
 	    return TRUE;
 	}
     } else {
+	int k;
+
 	/* break is a special case */
 	if (*keybuf == '\0') {
 	    fresh_line();
@@ -422,7 +425,7 @@ found_exit(char *keybuf, int hx, int cc)
     } else {
 	s = expand_to(keybuf, 10);
     }
-    sprintf(temp, "%s Unknown", s);
+    sprintf(temp, "%.80s Unknown", s);
     put_str(temp);
     for (j = 0; j < MAX_FK_UNK; j++) {
 	if (j == funk) {
@@ -499,6 +502,18 @@ tty_meta_prep(void)
     return 1;
 }
 
+static int
+put_reply(int col, int limit, const char *reply)
+{
+    int step = (int) strlen(reply);
+    if ((col += step) >= limit) {
+	put_crlf();
+	col = step;
+    }
+    put_str(reply);
+    return col;
+}
+
 /*
 **	funkey_meta(test_list, status, ch)
 **
@@ -510,10 +525,11 @@ funkey_meta(
 	       int *state,
 	       int *ch)
 {
-    int i, j, k, len;
-    char outbuf[256];
-
     if (has_meta_key) {
+	int i, j, k, len, pass;
+	const char *s;
+	char outbuf[256];
+
 	put_crlf();
 	if (char_mask != ALLOW_PARITY) {
 	    if (tty_meta_prep()) {
@@ -522,31 +538,45 @@ funkey_meta(
 		put_crlf();
 	    }
 	}
-	ptext("Begin meta key test. (km) (smm) (rmm)  Hit any key");
-	ptext(" with the meta key.  The character will be");
-	ptext(" displayed in hex.  If the meta key is working");
-	ptext(" then the most significant bit will be set.  Type");
-	ptextln(" 'end' to exit.");
-	tty_raw(1, ALLOW_PARITY);
-	tc_putp(meta_on);
+	for (pass = 0; pass < 2; ++pass) {
+	    if (pass) {
+		ptext("Continue meta key test.");
+	    } else {
+		ptext("Begin meta key test. (km) (smm) (rmm)");
+	    }
+	    ptext("  Hit any key while pressing the meta key.");
+	    ptext("  The character will be displayed in hex.");
+	    if (pass) {
+		ptext("  Meta mode is off;");
+		ptext(" the meta key should have no effect.");
+	    } else {
+		ptext("  If the meta key is working,");
+		ptext(" then the most significant bit will be set.");
+	    }
+	    put_crlf();
+	    ptextln("Type 'end' to exit.");
+	    tty_raw(1, ALLOW_PARITY);
+	    tc_putp(meta_on);
 
-	for (i = j = k = len = 0; i != 'e' || j != 'n' || k != 'd';) {
-	    i = j;
-	    j = k;
-	    k = getchp(ALLOW_PARITY);
-	    if (k == EOF) {
-		break;
+	    for (i = j = k = len = 0; i != 'e' || j != 'n' || k != 'd';) {
+		i = j;
+		j = k;
+		k = getchp(ALLOW_PARITY);
+		if (k == EOF) {
+		    break;
+		}
+		sprintf(outbuf, "%02X ", k);
+		len = put_reply(len, columns, outbuf);
+		if ((s = unctrl((chtype) k)) != NULL) {
+		    sprintf(outbuf, "(%s) ", s);
+		    len = put_reply(len, columns, outbuf);
+		}
+		k &= STRIP_PARITY;
 	    }
-	    if ((len += 3) >= columns) {
-		put_crlf();
-		len = 3;
-	    }
-	    sprintf(outbuf, "%02X ", k);
-	    put_str(outbuf);
-	    k &= STRIP_PARITY;
+	    if (!pass)
+		tc_putp(meta_off);
+	    put_crlf();
 	}
-	tc_putp(meta_off);
-	put_crlf();
 	tty_set();
 	put_crlf();
     } else {
@@ -566,13 +596,13 @@ funkey_label(
 		int *state,
 		int *ch)
 {
-    int i;
-    char outbuf[256];
-    int my_width = label_width;
-
     if (num_labels == -1) {
 	ptextln("Your terminal has no labels. (nlab)");
     } else {
+	int i;
+	int my_width = label_width;
+	char outbuf[256];
+
 	sprintf(temp,
 		"Your terminal has %d labels (nlab) that are %d characters wide (lw) and %d lines high (lh)",
 		num_labels, my_width, label_height);
@@ -609,11 +639,11 @@ funkey_prog(
 	       int *state,
 	       int *ch)
 {
-    int i, fk;
-    char mm[256];
-
-    fk = 1;			/* use function key 1 for now */
     if (pkey_xmit) {
+	int i;
+	int fk = 1;
+	char mm[256];
+
 	/* test program function key */
 	sprintf(temp,
 		"(pfx) Set function key %d to transmit abc\\n", fk);
@@ -659,15 +689,14 @@ funkey_local(
 		int *state,
 		int *ch)
 {
-    int fk;
-
-    fk = 1;
     if (pkey_local) {
+	int fk = 1;
+
 	/* test local function key */
 	sprintf(temp,
 		"(pfloc) Set function key %d to execute a clear and print \"Done!\"", fk);
 	ptextln(temp);
-	sprintf(temp, "%sDone!", liberated(clear_screen));
+	sprintf(temp, "%.*sDone!", TEMP_SIZE - 10, liberated(clear_screen));
 	tc_putp(TPARM_2(pkey_local, fk, temp));
 	sprintf(temp, "Hit function key %d.  Then hit return.", fk);
 	ptextln(temp);
@@ -739,11 +768,13 @@ static void
 line_pattern(void)
 {				/* put up a pattern that will help count the
 				   number of lines */
-    int i, j;
+    int i;
 
     put_clear();
     if (over_strike) {
 	for (i = 0; i < 100; i++) {
+	    int j;
+
 	    if (i) {
 		put_crlf();
 	    }
@@ -813,7 +844,7 @@ tools_report(
 		int *state GCC_UNUSED,
 		int *pch GCC_UNUSED)
 {
-    int i, j, ch, crp, crx, high_bit, save_scan_mode, hex_display;
+    int i, j, crp, crx, high_bit, save_scan_mode, hex_display;
     char buf[1024];
     char txt[8];
 
@@ -831,7 +862,7 @@ tools_report(
     save_scan_mode = scan_mode;
     tty_raw(1, char_mask);
     for (i = crp = high_bit = 0;;) {
-	ch = getchp(char_mask);
+	int ch = getchp(char_mask);
 	if (ch == EOF) {
 	    break;
 	}
@@ -848,14 +879,14 @@ tools_report(
 	    if (hex_display) {
 		ptext(hex_expand_to(&buf[i - 1], 3));
 	    } else {
-		tc_putch(ch | high_bit);
+		tc_putch((TC_PUTCH) (ch | high_bit));
 	    }
 	} else if (ch == '\n' || ch == '\r') {
 	    /* status report test */
 	    put_crlf();
 	    crp = 0;
 	} else if (crp++ < crx) {
-	    tc_putch(ch | high_bit);
+	    tc_putch((TC_PUTCH) (ch | high_bit));
 	} else {
 	    put_str(expand(&buf[i - 1]));
 	}
