@@ -1,12 +1,12 @@
 /*
-** Copyright (C) 1991, 1997-2013,2017 Free Software Foundation, Inc.
+** Copyright 2017-2019,2020 Thomas E. Dickey
+** Copyright 1997-2013,2017 Free Software Foundation, Inc.
 **
 ** This file is part of TACK.
 **
 ** TACK is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2, or (at your option)
-** any later version.
+** the Free Software Foundation, version 2.
 **
 ** TACK is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,15 +20,19 @@
 */
 /* initialization and wrapup code */
 
+#ifdef _ALL_SOURCE
+#define _ACS_COMPAT_CODE	/* hack for AIX */
+#endif
+
 #include <tack.h>
 
-MODULE_ID("$Id: init.c,v 1.29 2017/07/25 22:58:47 tom Exp $")
+MODULE_ID("$Id: init.c,v 1.41 2020/02/20 21:54:47 tom Exp $")
 
 FILE *debug_fp;
-char temp[1024];
+char temp[TEMP_SIZE];
 char *tty_basename = NULL;
 
-#ifndef HAVE_CURSES_DATA_BOOLNAMES
+#if !(defined(HAVE_CURSES_DATA_BOOLNAMES) || defined(DECL_CURSES_DATA_BOOLNAMES))
 char **boolnames;
 char **numnames;
 char **strnames;
@@ -37,7 +41,7 @@ size_t max_numbers;
 size_t max_strings;
 #endif
 
-void
+static void
 put_name(const char *cap, const char *name)
 {				/* send the cap name followed by the cap */
     if (cap) {
@@ -75,14 +79,13 @@ reset_init(void)
     /* run the reset file */
     if (reset_file && reset_file[0]) {
 	FILE *fp;
-	int ch;
 
 	can_test("rf", FLAG_TESTED);
 	if ((fp = fopen(reset_file, "r"))) {	/* send the reset file */
 	    sprintf(temp, " (rf) %s", reset_file);
 	    ptextln(temp);
 	    while (1) {
-		ch = getc(fp);
+		int ch = getc(fp);
 		if (ch == EOF)
 		    break;
 		put_this(ch);
@@ -114,14 +117,13 @@ reset_init(void)
     /* run the initialization file */
     if (init_file && init_file[0]) {
 	FILE *fp;
-	int ch;
 
 	can_test("if", FLAG_TESTED);
 	if ((fp = fopen(init_file, "r"))) {	/* send the init file */
 	    sprintf(temp, " (if) %s", init_file);
 	    ptextln(temp);
 	    while (1) {
-		ch = getc(fp);
+		int ch = getc(fp);
 		if (ch == EOF)
 		    break;
 		put_this(ch);
@@ -199,14 +201,16 @@ ask_infocmp(void)
     char *result = 0;
     size_t need = strlen(tty_basename) + 20;
     char *command = malloc(need);
-    FILE *pp;
-    char buffer[BUFSIZ];
-    char *s, *t;
 
     if (command != 0) {
+	FILE *pp;
+
 	sprintf(command, "infocmp -1 \"%s\"", tty_basename);
 	if ((pp = popen(command, "r")) != 0) {
-	    if (fgets(buffer, sizeof(buffer) - 1, pp) != 0
+	    char buffer[BUFSIZ];
+	    char *s, *t;
+
+	    if (fgets(buffer, (int) (sizeof(buffer) - 1), pp) != 0
 		&& *buffer == '#'
 		&& ((t = strstr(buffer, " file: "))
 		    || (t = strstr(buffer, " file "))
@@ -217,7 +221,7 @@ ask_infocmp(void)
 		s = strchr(t + 1, ' ');
 		result = strdup(s + 1);
 	    }
-#ifndef HAVE_CURSES_DATA_BOOLNAMES
+#if !(defined(HAVE_CURSES_DATA_BOOLNAMES) || defined(DECL_CURSES_DATA_BOOLNAMES))
 	    if (result) {
 		int max_b = 200;
 		int max_n = 200;
@@ -272,20 +276,30 @@ ask_infocmp(void)
 static void
 init_acs(void)
 {
-    int s, d;
     const char *value = acs_chars;
 
-#ifdef NUM_ACS			/* NetBSD curses */
-#ifndef acs_map
+#ifdef HAVE_CURSES_DATA_ACS_MAP
+#elif defined(HAVE_CURSES_DATA__ACS_MAP)
 #define acs_map _acs_map
+#elif defined(HAVE_CURSES_DATA___ACS_MAP)
+#define acs_map __acs_map
+#elif !defined(DECL_CURSES_DATA_ACS32MAP)
+#define NEED_ACS_MAP
 #endif
-#elif !defined(NCURSES_VERSION)
+
+    /*
+     * HPUX and AIX, which do #define's for acs_map, make those arrays.
+     * Solaris makes it a pointer (which we allocate).
+     */
+#if (defined(HAVE_CURSES_DATA_ACS_MAP) || defined(NEED_ACS_MAP)) && ! (defined(NUM_ACS) || defined(NCURSES_VERSION))
     if (acs_map == 0) {
-	acs_map = calloc(256, sizeof(acs_map[0]));
+	acs_map = calloc((size_t) 256, sizeof(acs_map[0]));
     }
 #endif
     if (value != 0) {
 	while (*value != '\0') {
+	    int s, d;
+
 	    if ((s = UChar(*value++)) == '\0')
 		break;
 	    if ((d = UChar(*value++)) == '\0')
@@ -472,9 +486,11 @@ bye_kids(int n)
     }
     if (debug_fp) {
 	fclose(debug_fp);
+	debug_fp = NULL;
     }
     if (log_fp) {
 	fclose(log_fp);
+	log_fp = NULL;
     }
     tty_reset();
     fclose(stdin);
@@ -484,3 +500,12 @@ bye_kids(int n)
 	sleep(1);
     ExitProgram(n);
 }
+
+const char *
+safe_tgets(NCURSES_CONST char *name)
+{
+    char *value = (char *) tigetstr(name);
+    if (!VALID_STRING(value))
+	value = 0;
+    return value;
+}
