@@ -1,12 +1,12 @@
 /*
-** Copyright (C) 1991, 1997-2010,2017 Free Software Foundation, Inc.
+** Copyright 2017,2020 Thomas E. Dickey
+** Copyright 1997-2010,2017 Free Software Foundation, Inc.
 **
 ** This file is part of TACK.
 **
 ** TACK is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2, or (at your option)
-** any later version.
+** the Free Software Foundation, version 2.
 **
 ** TACK is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,154 +20,17 @@
 */
 
 #include <tack.h>
+#include <time.h>
 
-MODULE_ID("$Id: pad.c,v 1.13 2017/07/23 16:53:27 tom Exp $")
+MODULE_ID("$Id: pad.c,v 1.18 2020/11/28 15:53:32 tom Exp $")
 
 /* test the pad counts on the terminal */
 
-static void pad_standard(TestList *, int *, int *);
-static void init_xon_xoff(TestList *, int *, int *);
-static void init_cup(TestList *, int *, int *);
-static void pad_rmxon(TestList *, int *, int *);
-static void pad_home1(TestList *, int *, int *);
-static void pad_home2(TestList *, int *, int *);
-static void pad_clear(TestList *, int *, int *);
-static void pad_ech(TestList *, int *, int *);
-static void pad_el1(TestList *, int *, int *);
-static void pad_el(TestList *, int *, int *);
-static void pad_smdc(TestList *, int *, int *);
-static void pad_dch(TestList *, int *, int *);
-static void pad_dch1(TestList *, int *, int *);
-static void pad_smir(TestList *, int *, int *);
-static void pad_ich(TestList *, int *, int *);
-static void pad_ich1(TestList *, int *, int *);
-static void pad_xch1(TestList *, int *, int *);
-static void pad_rep(TestList *, int *, int *);
-static void pad_cup(TestList *, int *, int *);
-static void pad_hd(TestList *, int *, int *);
-static void pad_hu(TestList *, int *, int *);
-static void pad_rin(TestList *, int *, int *);
-static void pad_il(TestList *, int *, int *);
-static void pad_indn(TestList *, int *, int *);
-static void pad_dl(TestList *, int *, int *);
-static void pad_xl(TestList *, int *, int *);
-static void pad_scrc(TestList *, int *, int *);
-static void pad_csrind(TestList *, int *, int *);
-static void pad_sccsrrc(TestList *, int *, int *);
-static void pad_csr_nel(TestList *, int *, int *);
-static void pad_csr_cup(TestList *, int *, int *);
-static void pad_ht(TestList *, int *, int *);
-static void pad_smso(TestList *, int *, int *);
-static void pad_smacs(TestList *, int *, int *);
-static void pad_crash(TestList *, int *, int *);
-
 /*
    Any command found in this list, executed from a "Done" prompt
    will force the default action to repeat rather than next.
 */
-const char *pad_repeat_test =
-{"ep-+<>"};
-/* *INDENT-OFF* */
-TestList pad_test_list[] = {
-    MY_EDIT_MENU
-    MY_PADS_MENU
-    {0, 0, 0, 0, "@) display statistics about the last test", dump_test_stats, 0},
-    {0, 0, 0, 0, "c) clear screen", menu_clear_screen, 0},
-    {0, 0, 0, 0, "i) send reset and init", menu_reset_init, 0},
-    {0, 0, 0, 0, txt_longer_test_time, longer_test_time, 0},
-    {0, 0, 0, 0, txt_shorter_test_time, shorter_test_time, 0},
-    {0, 0, 0, 0, txt_longer_augment, longer_augment, 0},
-    {0, 0, 0, 0, txt_shorter_augment, shorter_augment, 0},
-    /***
-       Phase 1: Test initialization and reset strings.
-
-       (rs1) (rs2) (rs3) (is1) (is2) (is3) are very difficult to test.
-       They have no defined output.  To make matters worse, the cap
-       builder could partition (rs1) (rs2) (rs3) by length, leaving the
-       terminal in some unknown state between (rs1) and (rs2) or between
-       (r2) and (rs3).  Some reset strings clear the screen when done.
-
-       We have no control over this.  The only thing we can do for
-       certain is to test the pad times by checking for overruns.
-    ***/
-    {MENU_NEXT, 3, "rs1", 0, 0, pad_standard, 0},
-    {MENU_NEXT, 3, "rs2", 0, 0, pad_standard, 0},
-    {MENU_NEXT, 3, "rs3", 0, 0, pad_standard, 0},
-    {MENU_NEXT | MENU_INIT, 0, 0, 0, 0, init_xon_xoff, 0},
-    {MENU_NEXT, 3, "is1", 0, 0, pad_standard, 0},
-    {MENU_NEXT, 3, "is2", 0, 0, pad_standard, 0},
-    {MENU_NEXT, 3, "is3", 0, 0, pad_standard, 0},
-    {MENU_NEXT, 3, "rmxon", "smxon", 0, pad_rmxon, 0},
-    {MENU_NEXT | MENU_INIT, 0, 0, 0, 0, init_cup, 0},
-    /*
-       Phase 2: Test home, screen clears and erases.
-    */
-    {MENU_NEXT, 0, "home", 0, 0, pad_home1, 0},
-    {MENU_NEXT, 0, "home) (nel", 0, 0, pad_home2, 0},
-    {MENU_NEXT | 1, 0, "clear", 0, 0, pad_clear, 0},
-    {MENU_NEXT | MENU_LM1, 0, "ed", 0, 0, pad_clear, 0},
-    {MENU_NEXT | MENU_80c, 0, "ech", 0, 0, pad_ech, 0},
-    {MENU_NEXT | MENU_80c, 0, "el1", "cub1 nel", 0, pad_el1, 0},
-    {MENU_NEXT | MENU_10c, 0, "el", "nel", 0, pad_el, 0},
-    /*
-       Phase 3: Character deletions and insertions
-    */
-    {MENU_NEXT, 0, "smdc) (rmdc", 0, 0, pad_smdc, 0},
-    {MENU_NEXT | MENU_80c, 0, "dch", "smdc rmdc", 0, pad_dch, 0},
-    {MENU_NEXT | MENU_80c, 0, "dch1", "smdc rmdc", 0, pad_dch1, 0},
-    {MENU_NEXT, 0, "smir) (rmir", 0, 0, pad_smir, 0},
-    {MENU_NEXT | MENU_90c, 0, "ich) (ip", "smir rmir", 0, pad_ich, 0},
-    {MENU_NEXT | MENU_90c, 0, "ich1) (ip", "smir rmir", 0, pad_ich1, 0},
-    {MENU_NEXT, 4, "ich1) (dch1", "smir rmir", 0, pad_xch1, 0},
-    {MENU_NEXT | MENU_90c, 0, "rep", 0, 0, pad_rep, 0},
-    /*
-       Phase 4: Test cursor addressing pads.
-    */
-    {MENU_NEXT, 0, "cup", 0, 0, pad_cup, 0},
-    /*
-       Phase 5: Test scrolling and cursor save/restore.
-    */
-    {MENU_NEXT, 0, "hd", 0, 0, pad_hd, 0},
-    {MENU_NEXT, 0, "hu", 0, 0, pad_hu, 0},
-    {MENU_NEXT | MENU_LM1 | 1, 0, "rin", 0, 0, pad_rin, 0},
-    {MENU_NEXT, 0, "ri", 0, 0, pad_rin, 0},
-    {MENU_NEXT | MENU_LM1 | 1, 0, "il", 0, 0, pad_il, 0},
-    {MENU_NEXT, 0, "il1", 0, 0, pad_il, 0},
-    {MENU_NEXT | MENU_LM1 | 1, 0, "indn", 0, 0, pad_indn, 0},
-    {MENU_NEXT, 0, "ind", 0, 0, pad_indn, 0},
-    {MENU_NEXT | MENU_LM1 | 1, 0, "dl", 0, 0, pad_dl, 0},
-    {MENU_NEXT, 0, "dl1", 0, 0, pad_dl, 0},
-    {MENU_NEXT, 0, "il1) (dl1", 0, 0, pad_xl, 0},
-    {MENU_NEXT, 0, "sc) (rc", 0, 0, pad_scrc, 0},
-    {MENU_NEXT | MENU_50l, 0, "csr) (ind", 0, 0, pad_csrind, 0},
-    {MENU_NEXT, 0, "sc) (csr) (rc", 0, 0, pad_sccsrrc, 0},
-    {MENU_NEXT, 0, "csr) (nel", "sc rc", 0, pad_csr_nel, 0},
-    {MENU_NEXT, 0, "csr) (cup", 0, 0, pad_csr_cup, 0},
-    /*
-       Phase 6: Test tabs.
-    */
-    {MENU_NEXT, 0, "ht", 0, 0, pad_ht, 0},
-    /*
-       Phase 7: Test character-set-switch pads.
-    */
-    {MENU_NEXT, 0, "smso) (rmso", 0, 0, pad_smso, 0},
-    {MENU_NEXT, 0, "smacs) (rmacs", 0, 0, pad_smacs, 0},
-    /*
-       Phase 8: Tests for miscellaneous mode-switch pads.
-    */
-    {MENU_NEXT, 3, "flash", 0, 0, pad_standard, 0},
-    {MENU_NEXT, 3, "smkx", 0, 0, pad_standard, 0},
-    {MENU_NEXT, 3, "rmkx", 0, 0, pad_standard, 0},
-    {MENU_NEXT, 3, "smm", 0, 0, pad_standard, 0},
-    {MENU_NEXT, 3, "rmm", 0, 0, pad_standard, 0},
-    /*
-       Phase 9: Test crash-and-burn properties of unpadded (clear).
-    */
-    {0, 0, "clear", "xon", "k) run clear test with no padding", pad_crash, 0},
-    {MENU_LAST, 0, 0, 0, 0, 0, 0}
-};
-/* *INDENT-ON* */
-
+const char *pad_repeat_test = "ep-+<>";
 /* globals */
 static int hzcc;		/* horizontal character count */
 char letter;			/* current character being displayed */
@@ -193,10 +56,12 @@ pad_standard(
 {
     const char *long_name;
     const char *cap;
-    int l = 2, i;
-    char tbuf[128];
 
     if ((cap = get_string_cap_byname(t->caps_done, &long_name))) {
+	int i;
+	int l = 2;
+	char tbuf[128];
+
 	sprintf(tbuf, "(%s) %s, start testing", t->caps_done,
 		long_name);
 	if (skip_pad_test(t, state, ch, tbuf)) {
@@ -213,6 +78,7 @@ pad_standard(
 	    tt_putp(cap);
 	    putchp(letter);
 	    i++;
+	    ENSURE_DELAY;
 	} while (still_testing());
 	pad_test_shutdown(t, 0);
 	if (l >= lines) {
@@ -305,7 +171,6 @@ pad_home1(
 	     int *state,
 	     int *ch)
 {
-    int j, k;
 
     if (can_go_home && auto_right_margin) {
 	/*
@@ -317,8 +182,12 @@ pad_home1(
 	}
 	pad_test_startup(1);
 	do {
+	    int j;
+
 	    go_home();
 	    for (j = 1; j < lines; j++) {
+		int k;
+
 		for (k = 0; k < columns; k++) {
 		    if (k & 0xF) {
 			put_this(letter);
@@ -329,6 +198,7 @@ pad_home1(
 		SLOW_TERMINAL_EXIT;
 	    }
 	    NEXT_LETTER;
+	    ENSURE_DELAY;
 	} while (still_testing());
 	pad_test_shutdown(t, 0);
 	ptext("All the dots should line up.  ");
@@ -348,8 +218,6 @@ pad_home2(
 	     int *state,
 	     int *ch)
 {
-    int j, k;
-
     if (can_go_home) {
 	if (skip_pad_test(t, state, ch,
 			  "(home) Home, (nel) newline start testing")) {
@@ -357,8 +225,12 @@ pad_home2(
 	}
 	pad_test_startup(1);
 	do {
+	    int j;
+
 	    go_home();
 	    for (j = 1; j < lines; j++) {
+		int k;
+
 		for (k = 2; k < columns; k++) {
 		    if (k & 0xF) {
 			put_this(letter);
@@ -370,6 +242,7 @@ pad_home2(
 		SLOW_TERMINAL_EXIT;
 	    }
 	    NEXT_LETTER;
+	    ENSURE_DELAY;
 	} while (still_testing());
 	pad_test_shutdown(t, 0);
 	ptext("All the dots should line up.  ");
@@ -513,6 +386,7 @@ pad_clear(
 		tt_tputs(clr_eos, repeats);
 	    }
 	    NEXT_LETTER;
+	    ENSURE_DELAY;
 	} while (still_testing());
 	pad_test_shutdown(t, 1);
 	ptext(end_message);
@@ -569,6 +443,7 @@ pad_ech(
 	putchp(letter);
 	put_crlf();
 	NEXT_LETTER;
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     ptext(all_lines);
@@ -621,6 +496,7 @@ pad_el1(
 	putchp(letter);
 	put_crlf();
 	NEXT_LETTER;
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     ptext(all_lines);
@@ -671,6 +547,7 @@ pad_el(
 	putchp(letter);
 	put_crlf();
 	NEXT_LETTER;
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     ptext(all_lines);
@@ -713,6 +590,7 @@ pad_smdc(
 	    tt_putp(exit_delete_mode);
 	    putchp(letter);
 	}
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     home_down();
@@ -765,6 +643,7 @@ pad_dch(
 	putchp(letter);
 	put_crlf();
 	NEXT_LETTER;
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     home_down();
@@ -823,6 +702,7 @@ pad_dch1(
 	putchp(letter);
 	put_crlf();
 	NEXT_LETTER;
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     ptext(all_lines);
@@ -865,6 +745,7 @@ pad_smir(
 	    tt_putp(exit_insert_mode);
 	    putchp(letter);
 	}
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     home_down();
@@ -920,6 +801,7 @@ pad_ich(
 	putchp(letter);
 	NEXT_LETTER;
 	put_crlf();
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     ptext(all_lines);
@@ -984,6 +866,7 @@ pad_ich1(
 	putchp(letter);
 	NEXT_LETTER;
 	put_crlf();
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     ptext(all_lines);
@@ -1022,6 +905,7 @@ pad_xch1(
     do {
 	tt_putp(insert_character);
 	tt_putp(delete_character);
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 1);
     ptextln(xch1);
@@ -1070,6 +954,7 @@ pad_rep(
 	}
 	put_crlf();
 	NEXT_LETTER;
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     ptextln(all_lines);
@@ -1129,6 +1014,7 @@ pad_cup(
 	    SLOW_TERMINAL_EXIT;
 	}
 	NEXT_LETTER;
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     tt_putparm(cursor_address, 1, line_count = r, char_count = c);
@@ -1177,6 +1063,7 @@ pad_hd(
 	    SLOW_TERMINAL_EXIT;
 	}
 	NEXT_LETTER;
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     pad_done_message(t, state, ch);
@@ -1225,6 +1112,7 @@ pad_hu(
 	}
 	go_home();
 	NEXT_LETTER;
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     pad_done_message(t, state, ch);
@@ -1242,7 +1130,6 @@ pad_rin(
 	   int *state,
 	   int *ch)
 {
-    int i;
     const char *start_message;
 
     if (t->flags & 1) {
@@ -1268,6 +1155,7 @@ pad_rin(
     if (skip_pad_test(t, state, ch, start_message)) {
 	return;
     }
+
     pad_test_startup(1);
     do {
 	sprintf(temp, "%d\r", test_complete);
@@ -1277,9 +1165,14 @@ pad_rin(
 	} else {
 	    tt_putparm(parm_rindex, repeats, repeats, 0);
 	}
+	ENSURE_DELAY;
     } while (still_testing());
+
     put_str("This line should be on the bottom.\r");
+
     if (scroll_reverse && augment == 1) {
+	int i;
+
 	for (i = 1; i < lines; i++) {
 	    tt_putp(scroll_reverse);
 	}
@@ -1306,7 +1199,6 @@ pad_il(
 	  int *state,
 	  int *ch)
 {
-    int i;
     const char *start_message;
 
     if (t->flags & 1) {
@@ -1332,6 +1224,7 @@ pad_il(
     if (skip_pad_test(t, state, ch, start_message)) {
 	return;
     }
+
     pad_test_startup(1);
     do {
 	sprintf(temp, "%d\r", test_complete);
@@ -1341,9 +1234,14 @@ pad_il(
 	} else {
 	    tt_putparm(parm_insert_line, repeats, repeats, 0);
 	}
+	ENSURE_DELAY;
     } while (still_testing());
+
     put_str("This line should be on the bottom.\r");
+
     if (insert_line && augment == 1) {
+	int i;
+
 	for (i = 1; i < lines; i++) {
 	    tt_putp(insert_line);
 	}
@@ -1370,7 +1268,6 @@ pad_indn(
 	    int *state,
 	    int *ch)
 {
-    int i;
     const char *start_message;
 
     if (t->flags & 1) {
@@ -1401,6 +1298,7 @@ pad_indn(
     if (skip_pad_test(t, state, ch, start_message)) {
 	return;
     }
+
     pad_test_startup(1);
     /* go to the bottom of the screen */
     home_down();
@@ -1412,9 +1310,14 @@ pad_indn(
 	} else {
 	    tt_putparm(parm_index, repeats, repeats, 0);
 	}
+	ENSURE_DELAY;
     } while (still_testing());
+
     put_str("This line should be on the top.\r");
+
     if (scroll_forward && augment == 1) {
+	int i;
+
 	for (i = 1; i < lines; i++) {
 	    put_ind();
 	}
@@ -1440,7 +1343,6 @@ pad_dl(
 	  int *state,
 	  int *ch)
 {
-    int i = 0;
     const char *start_message;
 
     if (t->flags & 1) {
@@ -1479,11 +1381,15 @@ pad_dl(
 	} else {
 	    tt_putparm(parm_delete_line, repeats, repeats, 0);
 	}
+	ENSURE_DELAY;
     } while (still_testing());
+
     home_down();
     put_str("This line should be on the top.");
     go_home();
+
     if (delete_line && augment == 1) {
+	int i;
 	for (i = 1; i < lines; i++) {
 	    tt_putp(delete_line);
 	}
@@ -1539,6 +1445,7 @@ pad_xl(
 	tt_putp(insert_line);
 	put_cr();
 	tt_putp(delete_line);
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     home_down();
@@ -1585,6 +1492,7 @@ pad_scrc(
 	    tt_putp(restore_cursor);
 	    putchp('X');
 	}
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     home_down();
@@ -1631,6 +1539,7 @@ pad_csrind(
 	sprintf(temp, "%d\r", test_complete);
 	put_str(temp);
 	put_ind();
+	ENSURE_DELAY;
     } while (still_testing());
     ptextln("(csr) is broken.");
     for (i = augment; i > 1; i--) {
@@ -1674,6 +1583,7 @@ pad_sccsrrc(
 	    tt_putp(restore_cursor);
 	    putchp('X');
 	}
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     home_down();
@@ -1717,6 +1627,7 @@ pad_csr_nel(
 	tt_putp(save_cursor);
 	tt_putparm(change_scroll_region, 1, 0, lines - 1);
 	tt_putp(restore_cursor);
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     put_str("  ");
@@ -1757,6 +1668,7 @@ pad_csr_cup(
 	}
 	tt_putparm(change_scroll_region, 1, 0, lines - 1);
 	tt_putparm(cursor_address, 1, lines - 1, (int) strlen(every_line));
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     put_str("  ");
@@ -1824,6 +1736,7 @@ pad_ht(
 	    }
 	    putln("N");
 	}
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
     ptextln("Letters on the screen other than Ns at the right margin indicate failure.");
@@ -1842,8 +1755,6 @@ pad_smso(
 	    int *state,
 	    int *ch)
 {
-    int i, j;
-
     if (!enter_standout_mode || !exit_standout_mode) {
 	CAP_NOT_FOUND;
 	ptext("(smso) (rmso) Enter/Exit-standout-mode not present.  ");
@@ -1862,6 +1773,8 @@ pad_smso(
      */
     pad_test_startup(1);
     do {
+	int i, j;
+
 	page_loop();
 	j = magic_cookie_glitch > 0 ? magic_cookie_glitch : 0;
 	for (i = 2 + j + j; i < columns;) {
@@ -1871,8 +1784,10 @@ pad_smso(
 	    put_mode(exit_standout_mode);
 	    putchp('X');
 	}
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
+
     home_down();
     ptext(above_line);
     pad_done_message(t, state, ch);
@@ -1890,7 +1805,6 @@ pad_smacs(
 	     int *state,
 	     int *ch)
 {
-    int i, j;
 
     /* test enter even if exit is missing */
     if (!enter_alt_charset_mode) {
@@ -1903,8 +1817,11 @@ pad_smacs(
 		      "(smacs) (rmacs) Enter/Exit-altcharset-mode start testing")) {
 	return;
     }
+
     pad_test_startup(1);
     do {
+	int i, j;
+
 	page_loop();
 	j = magic_cookie_glitch > 0 ? magic_cookie_glitch : 0;
 	for (i = 2 + j + j; i < columns;) {
@@ -1914,8 +1831,10 @@ pad_smacs(
 	    put_mode(exit_alt_charset_mode);
 	    putchp(letter);
 	}
+	ENSURE_DELAY;
     } while (still_testing());
     pad_test_shutdown(t, 0);
+
     home_down();
     ptext("Every other character is from the alternate character set.  ");
     pad_done_message(t, state, ch);
@@ -1952,8 +1871,110 @@ pad_crash(
     do {
 	put_str("Erase this!");
 	tt_putp(clear_screen);
+	ENSURE_DELAY;
     } while (still_testing());
     ChangeTermInfo(xon_xoff, save_xon_xoff);
     pad_test_shutdown(t, 1);
     pad_done_message(t, state, ch);
 }
+/* *INDENT-OFF* */
+
+TestList pad_test_list[] = {
+    MY_EDIT_MENU
+    MY_PADS_MENU
+    {0, 0, 0, 0, "@) display statistics about the last test", dump_test_stats, 0},
+    {0, 0, 0, 0, "c) clear screen", menu_clear_screen, 0},
+    {0, 0, 0, 0, "i) send reset and init", menu_reset_init, 0},
+    {0, 0, 0, 0, txt_longer_test_time, longer_test_time, 0},
+    {0, 0, 0, 0, txt_shorter_test_time, shorter_test_time, 0},
+    {0, 0, 0, 0, txt_longer_augment, longer_augment, 0},
+    {0, 0, 0, 0, txt_shorter_augment, shorter_augment, 0},
+    /***
+       Phase 1: Test initialization and reset strings.
+
+       (rs1) (rs2) (rs3) (is1) (is2) (is3) are very difficult to test.
+       They have no defined output.  To make matters worse, the cap
+       builder could partition (rs1) (rs2) (rs3) by length, leaving the
+       terminal in some unknown state between (rs1) and (rs2) or between
+       (r2) and (rs3).  Some reset strings clear the screen when done.
+
+       We have no control over this.  The only thing we can do for
+       certain is to test the pad times by checking for overruns.
+    ***/
+    {MENU_NEXT, 3, "rs1", 0, 0, pad_standard, 0},
+    {MENU_NEXT, 3, "rs2", 0, 0, pad_standard, 0},
+    {MENU_NEXT, 3, "rs3", 0, 0, pad_standard, 0},
+    {MENU_NEXT | MENU_INIT, 0, 0, 0, 0, init_xon_xoff, 0},
+    {MENU_NEXT, 3, "is1", 0, 0, pad_standard, 0},
+    {MENU_NEXT, 3, "is2", 0, 0, pad_standard, 0},
+    {MENU_NEXT, 3, "is3", 0, 0, pad_standard, 0},
+    {MENU_NEXT, 3, "rmxon", "smxon", 0, pad_rmxon, 0},
+    {MENU_NEXT | MENU_INIT, 0, 0, 0, 0, init_cup, 0},
+    /*
+       Phase 2: Test home, screen clears and erases.
+    */
+    {MENU_NEXT, 0, "home", 0, 0, pad_home1, 0},
+    {MENU_NEXT, 0, "home) (nel", 0, 0, pad_home2, 0},
+    {MENU_NEXT | 1, 0, "clear", 0, 0, pad_clear, 0},
+    {MENU_NEXT | MENU_LM1, 0, "ed", 0, 0, pad_clear, 0},
+    {MENU_NEXT | MENU_80c, 0, "ech", 0, 0, pad_ech, 0},
+    {MENU_NEXT | MENU_80c, 0, "el1", "cub1 nel", 0, pad_el1, 0},
+    {MENU_NEXT | MENU_10c, 0, "el", "nel", 0, pad_el, 0},
+    /*
+       Phase 3: Character deletions and insertions
+    */
+    {MENU_NEXT, 0, "smdc) (rmdc", 0, 0, pad_smdc, 0},
+    {MENU_NEXT | MENU_80c, 0, "dch", "smdc rmdc", 0, pad_dch, 0},
+    {MENU_NEXT | MENU_80c, 0, "dch1", "smdc rmdc", 0, pad_dch1, 0},
+    {MENU_NEXT, 0, "smir) (rmir", 0, 0, pad_smir, 0},
+    {MENU_NEXT | MENU_90c, 0, "ich) (ip", "smir rmir", 0, pad_ich, 0},
+    {MENU_NEXT | MENU_90c, 0, "ich1) (ip", "smir rmir", 0, pad_ich1, 0},
+    {MENU_NEXT, 4, "ich1) (dch1", "smir rmir", 0, pad_xch1, 0},
+    {MENU_NEXT | MENU_90c, 0, "rep", 0, 0, pad_rep, 0},
+    /*
+       Phase 4: Test cursor addressing pads.
+    */
+    {MENU_NEXT, 0, "cup", 0, 0, pad_cup, 0},
+    /*
+       Phase 5: Test scrolling and cursor save/restore.
+    */
+    {MENU_NEXT, 0, "hd", 0, 0, pad_hd, 0},
+    {MENU_NEXT, 0, "hu", 0, 0, pad_hu, 0},
+    {MENU_NEXT | MENU_LM1 | 1, 0, "rin", 0, 0, pad_rin, 0},
+    {MENU_NEXT, 0, "ri", 0, 0, pad_rin, 0},
+    {MENU_NEXT | MENU_LM1 | 1, 0, "il", 0, 0, pad_il, 0},
+    {MENU_NEXT, 0, "il1", 0, 0, pad_il, 0},
+    {MENU_NEXT | MENU_LM1 | 1, 0, "indn", 0, 0, pad_indn, 0},
+    {MENU_NEXT, 0, "ind", 0, 0, pad_indn, 0},
+    {MENU_NEXT | MENU_LM1 | 1, 0, "dl", 0, 0, pad_dl, 0},
+    {MENU_NEXT, 0, "dl1", 0, 0, pad_dl, 0},
+    {MENU_NEXT, 0, "il1) (dl1", 0, 0, pad_xl, 0},
+    {MENU_NEXT, 0, "sc) (rc", 0, 0, pad_scrc, 0},
+    {MENU_NEXT | MENU_50l, 0, "csr) (ind", 0, 0, pad_csrind, 0},
+    {MENU_NEXT, 0, "sc) (csr) (rc", 0, 0, pad_sccsrrc, 0},
+    {MENU_NEXT, 0, "csr) (nel", "sc rc", 0, pad_csr_nel, 0},
+    {MENU_NEXT, 0, "csr) (cup", 0, 0, pad_csr_cup, 0},
+    /*
+       Phase 6: Test tabs.
+    */
+    {MENU_NEXT, 0, "ht", 0, 0, pad_ht, 0},
+    /*
+       Phase 7: Test character-set-switch pads.
+    */
+    {MENU_NEXT, 0, "smso) (rmso", 0, 0, pad_smso, 0},
+    {MENU_NEXT, 0, "smacs) (rmacs", 0, 0, pad_smacs, 0},
+    /*
+       Phase 8: Tests for miscellaneous mode-switch pads.
+    */
+    {MENU_NEXT, 3, "flash", 0, 0, pad_standard, 0},
+    {MENU_NEXT, 3, "smkx", 0, 0, pad_standard, 0},
+    {MENU_NEXT, 3, "rmkx", 0, 0, pad_standard, 0},
+    {MENU_NEXT, 3, "smm", 0, 0, pad_standard, 0},
+    {MENU_NEXT, 3, "rmm", 0, 0, pad_standard, 0},
+    /*
+       Phase 9: Test crash-and-burn properties of unpadded (clear).
+    */
+    {0, 0, "clear", "xon", "k) run clear test with no padding", pad_crash, 0},
+    {MENU_LAST, 0, 0, 0, 0, 0, 0}
+};
+/* *INDENT-ON* */
