@@ -1,12 +1,12 @@
 /*
-** Copyright (C) 1991, 1997-2012,2017 Free Software Foundation, Inc.
+** Copyright 2017,2020 Thomas E. Dickey
+** Copyright 1997-2012,2017 Free Software Foundation, Inc.
 ** 
 ** This file is part of TACK.
 ** 
 ** TACK is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2, or (at your option)
-** any later version.
+** the Free Software Foundation, version 2.
 ** 
 ** TACK is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -22,7 +22,7 @@
 #include <tack.h>
 #include <time.h>
 
-MODULE_ID("$Id: sync.c,v 1.14 2017/07/23 16:08:17 tom Exp $")
+MODULE_ID("$Id: sync.c,v 1.17 2020/02/02 14:47:18 tom Exp $")
 
 /* terminal-synchronization and performance tests */
 
@@ -73,10 +73,12 @@ static char tty_ACK[TTY_ACK_SIZE];	/* ACK response, set by tty_sync_error() */
 int
 tty_sync_error(void)
 {
-    int ch, trouble, ack;
+    int trouble;
 
     trouble = FALSE;
     for (;;) {
+	int ch, ack;
+
 	tt_putp(tty_ENQ);	/* send ENQ */
 	ch = getnext(STRIP_PARITY);
 	event_start(TIME_SYNC);	/* start the timer */
@@ -139,7 +141,7 @@ flush_input(void)
 static void
 probe_enq_ok(void)
 {
-    int tc, len, ulen;
+    int len;
 
     put_str("Testing ENQ/ACK, standby...");
     fflush(stdout);
@@ -191,10 +193,10 @@ probe_enq_ok(void)
 	ACK_length = 4096;
 	return;
     }
-    tc = tty_ACK[len - 1];
 #ifdef user8
     if (user8) {
-	ulen = (int) strlen(user8);
+	int ulen = (int) strlen(user8);
+	int tc = tty_ACK[len - 1];
 	if (tc == user8[ulen - 1]) {
 	    /* ANSI style acknowledge string */
 	    ACK_terminator = tc;
