@@ -1,12 +1,12 @@
 /*
-** Copyright (C) 1991, 1997-2015,2017 Free Software Foundation, Inc.
+** Copyright 2017-2022,2023 Thomas E. Dickey
+** Copyright 1997-2015,2017 Free Software Foundation, Inc.
 **
 ** This file is part of TACK.
 **
 ** TACK is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2, or (at your option)
-** any later version.
+** the Free Software Foundation, version 2.
 **
 ** TACK is distributed in the hope that it will be useful,
 ** but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,7 +19,7 @@
 ** Boston, MA 02110-1301, USA
 */
 
-/* $Id: tack.h,v 1.64 2017/07/26 08:08:21 tom Exp $ */
+/* $Id: tack.h,v 1.95 2023/02/02 00:33:41 tom Exp $ */
 
 #ifndef NCURSES_TACK_H_incl
 #define NCURSES_TACK_H_incl 1
@@ -27,14 +27,12 @@
 /* terminfo action checker include file */
 
 #define MAJOR_VERSION 1
-#define MINOR_VERSION 8
-#define PATCH_VERSION 20170726
+#define MINOR_VERSION 9
+#define PATCH_VERSION 20230201
 
 #ifdef HAVE_CONFIG_H
 #include <ncurses_cfg.h>
 #else
-#define GCC_UNUSED		/*nothing */
-#define GCC_NORETURN		/*nothing */
 #define HAVE_GETTIMEOFDAY 1
 #define HAVE_SELECT 1
 #define HAVE_SYS_TIME_H 1
@@ -44,12 +42,24 @@
 #define BROKEN_LINKER 0
 #endif
 
+#ifndef GCC_NORETURN
+#define GCC_NORETURN		/*nothing */
+#endif
+
+#ifndef GCC_UNUSED
+#define GCC_UNUSED		/*nothing */
+#endif
+
 #ifndef HAVE_LONG_FILE_NAMES
 #define HAVE_LONG_FILE_NAMES 0
 #endif
 
 #ifndef NCURSES_CONST
+#ifdef NCURSES_VERSION
 #define NCURSES_CONST const
+#else
+#define NCURSES_CONST		/*nothing */
+#endif
 #endif
 
 #ifndef NO_LEAKS
@@ -77,9 +87,11 @@
 
 #include <curses.h>
 
-#ifdef NCURSES_VERSION
+#if defined(NCURSES_VERSION) && defined(HAVE_TERM_ENTRY_H)
 #include <term_entry.h>
+#define TACK_CAN_EDIT 1
 #else
+#define TACK_CAN_EDIT 0
 #include <term.h>
 #include <termios.h>
 #define TTY struct termios
@@ -119,10 +131,10 @@
 extern void tack_edit_leaks(void);
 extern void tack_fun_leaks(void);
 #ifdef HAVE__NC_FREE_TINFO
-extern void _nc_free_tinfo(int) GCC_NORETURN;
+extern GCC_NORETURN void _nc_free_tinfo(int);
 #endif
 #ifndef ExitProgram
-extern void ExitProgram(int) GCC_NORETURN;
+extern GCC_NORETURN void ExitProgram(int);
 #endif
 #else
 #define ExitProgram(code) exit(code)
@@ -140,7 +152,35 @@ extern void ExitProgram(int) GCC_NORETURN;
 
 #include <tackgen.h>
 
-#ifdef NCURSES_VERSION
+#ifdef DECL_CURSES_DATA_BOOLNAMES
+#undef boolnames
+extern char *boolnames[];
+extern size_t max_booleans;
+
+#undef numnames
+extern char *numnames[];
+extern size_t max_numbers;
+
+#undef strnames
+extern char *strnames[];
+extern size_t max_strings;
+
+#undef boolfnames
+extern char *boolfnames[];
+
+#undef numfnames
+extern char *numfnames[];
+
+#undef strfnames
+extern char *strfnames[];
+
+#endif
+
+#define ABSENT_STRING		(char *)0
+#define CANCELLED_STRING	(char *)(-1)
+#define VALID_STRING(s)  ((s) != CANCELLED_STRING && (s) != ABSENT_STRING)
+
+#if TACK_CAN_EDIT
 #define CUR_TP      ((TERMTYPE *)(cur_term))
 #define MAX_BOOLEAN BOOLCOUNT	/* NUM_BOOLEANS(CUR_TP) */
 #define MAX_NUMBERS NUMCOUNT	/* NUM_NUMBERS(CUR_TP) */
@@ -165,10 +205,11 @@ typedef enum {
 } NAME_TYPE;
 
 /* see ncurses' nc_tparm.h */
+#define TPARM_FMT NCURSES_CONST char *
 #define TPARM_ARG long
 #define TPARM_N(n) (TPARM_ARG)(n)
 
-#define TPARM_9(a,b,c,d,e,f,g,h,i,j) tparm(a,TPARM_N(b),TPARM_N(c),TPARM_N(d),TPARM_N(e),TPARM_N(f),TPARM_N(g),TPARM_N(h),TPARM_N(i),TPARM_N(j))
+#define TPARM_9(a,b,c,d,e,f,g,h,i,j) tparm((TPARM_FMT)a,TPARM_N(b),TPARM_N(c),TPARM_N(d),TPARM_N(e),TPARM_N(f),TPARM_N(g),TPARM_N(h),TPARM_N(i),TPARM_N(j))
 #define TPARM_8(a,b,c,d,e,f,g,h,i) TPARM_9(a,b,c,d,e,f,g,h,i,0)
 #define TPARM_7(a,b,c,d,e,f,g,h) TPARM_8(a,b,c,d,e,f,g,h,0)
 #define TPARM_6(a,b,c,d,e,f,g) TPARM_7(a,b,c,d,e,f,g,0)
@@ -177,15 +218,20 @@ typedef enum {
 #define TPARM_3(a,b,c,d) TPARM_4(a,b,c,d,0)
 #define TPARM_2(a,b,c) TPARM_3(a,b,c,0)
 #define TPARM_1(a,b) TPARM_2(a,b,0)
-#define TPARM_1(a,b) TPARM_2(a,b,0)
 #define TPARM_0(a) TPARM_1(a,0)
 
 #define UChar(c)    ((unsigned char)(c))
 
+#define NAME_SIZE 32
+#define TEMP_SIZE 1024
+
+#define LOG_FILENAME "tack.log"
+#define DBG_FILENAME "debug.log"
+
 extern FILE *log_fp;
 extern FILE *debug_fp;
 extern int debug_level;
-extern char temp[];
+extern char temp[TEMP_SIZE];
 extern char *tty_basename;
 extern char tty_shortname[];
 
@@ -214,6 +260,7 @@ extern int raw_characters_sent;	/* Total output characters */
 
 /* definitions for pad.c */
 
+#define ENSURE_DELAY  if (!tt_delay_used) napms(10)
 #define EXIT_CONDITION (no_alarm_event && (tt_delay_used < tt_delay_max))
 #define SLOW_TERMINAL_EXIT if (!test_complete && !EXIT_CONDITION) { break; }
 #define CAP_NOT_FOUND if (auto_pad_mode) return
@@ -397,7 +444,7 @@ typedef struct test_menu {
 #define REQUEST_PROMPT 256
 
 /* tack.c */
-#ifdef NCURSES_VERSION
+#if TACK_CAN_EDIT
 extern TestMenu edit_menu;
 #define MY_EDIT_MENU	{0, 0, 0, 0, "e) edit terminfo", 0, &edit_menu},
 #else
@@ -405,7 +452,7 @@ extern TestMenu edit_menu;
 #endif
 #ifdef DEBUG
 #define TACKMSG(p) p
-extern void TackMsg(const char *,...) GCC_PRINTFLIKE(1,2);
+extern void TackMsg(const char *, ...) GCC_PRINTFLIKE(1,2);
 #else
 #define TACKMSG(p)		/* nothing */
 #endif
@@ -420,10 +467,12 @@ extern char *hex_expand_to(char *, int);
 extern char *print_expand(char *);
 extern int getchp(int);
 extern int getnext(int);
+extern int log_chr(FILE *, int, int);
 extern int tc_putp(const char *);
 extern int wait_here(void);
 extern void go_home(void);
 extern void home_down(void);
+extern void log_str(FILE *, const char *);
 extern void maybe_wait(int);
 extern void ptext(const char *);
 extern void ptextln(const char *);
@@ -431,7 +480,6 @@ extern void put_clear(void);
 extern void put_columns(const char *, int, int);
 extern void put_cr(void);
 extern void put_crlf(void);
-extern void put_dec(char *, int);
 extern void put_ind(void);
 extern void put_lf(void);
 extern void put_newlines(int);
@@ -440,7 +488,6 @@ extern void put_this(int);
 extern void putchp(int);
 extern void putln(const char *);
 extern void read_string(char *, size_t);
-extern void three_digit(char *, int);
 extern void tt_putp(const char *);
 extern void tt_putparm(NCURSES_CONST char *, int, int, int);
 extern void tt_tputs(const char *, int);
@@ -494,7 +541,7 @@ extern void tools_charset(TestList *, int *, int *);
 extern void tools_sgr(TestList *, int *, int *);
 
 /* edit.c */
-#ifdef NCURSES_VERSION
+#if TACK_CAN_EDIT
 extern TestMenu change_pad_menu;
 extern TestList edit_test_list[];
 #define MY_PADS_MENU	{0, 0, 0, 0, "p) change padding", 0, &change_pad_menu},
@@ -519,9 +566,9 @@ extern int tty_meta_prep(void);
 extern void tools_report(TestList *, int *, int *);
 
 /* init.c */
+extern const char *safe_tgets(NCURSES_CONST char *);
 extern void reset_init(void);
 extern void display_basic(void);
-extern void put_name(const char *, const char *);
 extern void charset_can_test(void);
 extern void curses_setup(char *);
 extern void bye_kids(int);
